"""
copyright ICSI
"""

import sys
import os
from utils import *
from preClusterer import *
from clusterer import *
from postClusterer import *
from homepage_finder import *
from makeWebpages import *

def pipeline(name):
    """
    split input name into first and last name
    """
    lastname = name.split()[-1]
    firstname = name.split()[0]

    #path to data for input name
    pathPrefix = "../data/"+name.replace(" ", "-")

    """
    load lists
    """
    homepages = getpossiblehomepages(firstname, lastname, pathPrefix)  #get list of homepages
    filenames = os.listdir(pathPrefix+"/html/")  #list of files
    jobsList = wordFileToList("../lists/jobs.txt")  #list of professions

    stopWordList = wordFileToList("../lists/stopwords.txt")  #list of stopwords
    stopWordList.extend(wordFileToList("../lists/extendedstopwords.txt"))
    stopWordList.append(firstname.lower())
    stopWordList.append(lastname.lower())

    """
    create variables to store cluster information
    """
    clusterIDToFileList = {}  #dict with key = clusterID, value = list of urls
    clusterIDToBigrams = {}  #dict with key = clusterID, value = bigrams dict
    clusterIDToWordSetList = {}  #dict with key = clusterID, value = list of word between barriers
    clusterIDToLinks = {}  #dict with key = clusterID, value = links dict
    clusterIDToEmployHist = {}  #dict with key = clusterID, value = employment history dict
    clusterIDToEduHist = {}  #dict with key = clusterID, value = education history dict
    clusterIDToEmails = {}  #dict with key = clusterID, value = emails dict
    clusterIDToMiddleName = {}  #dict with key = clusterID, value = middlename dict
    clusterIDToRepFile = {}  #dict with key = clusterID, value = representative file dict
    clusterIDToImageTag = {} #dict with key = clusterID, value = image tag
    clusterIDToLinkTitles = {}  #dict with key = clusterID, value = link titles dict
    clusterIDToSelectedFiles = {}  #dict with key = clusterID, value = selected files
    fileNameToRank = {}  #dict with key = url, value = querry rank
    ranksList = wordFileToList(pathPrefix+"/rankings.txt")
    for rank in ranksList:
        rankList = rank.split()
        fileNameToRank[rankList[0]] = int(rankList[1])

    """
    Extract information from files
    """
    preCluster(filenames, pathPrefix, homepages, firstname, lastname, stopWordList, clusterIDToFileList, clusterIDToBigrams, \
               clusterIDToWordSetList, clusterIDToLinks, clusterIDToEmployHist, clusterIDToEduHist, clusterIDToEmails, clusterIDToMiddleName)
    
    """
    Cluster
    """
    minCommonBigrams = 5  #min # of bigrams needed to combine clusters
    minCommonBigramMinSum = 10  #min common bigram counts needed to combine clusters
    minCommonLinks = 2  #min # of links needed to combine clusters
    minCommonLinkMinSum = 2  #min common link counts needed to combine clusters
    cluster(minCommonBigrams, minCommonBigramMinSum, minCommonLinks, minCommonLinkMinSum, clusterIDToFileList, clusterIDToBigrams, \
            clusterIDToWordSetList, clusterIDToLinks, clusterIDToEmployHist, clusterIDToEduHist, clusterIDToEmails, clusterIDToMiddleName,\
            lastname)

    """
    Combine common information, remove bad clusters, find representative files, extract images, extract link titles, and get selected files
    """

    numFeaturesToShow = 5

    removedFiles = postCluster(clusterIDToFileList, clusterIDToBigrams, clusterIDToWordSetList, clusterIDToRepFile, clusterIDToImageTag, \
                               clusterIDToSelectedFiles, clusterIDToLinkTitles, fileNameToRank, firstname, lastname, pathPrefix, \
                               numFeaturesToShow, homepages)

    """
    Make the webpages
    """
    #make pretty print page
    writeStringToFile(makeHTML(clusterIDToBigrams, clusterIDToEmployHist, clusterIDToEduHist, clusterIDToEmails, \
                               clusterIDToMiddleName, clusterIDToSelectedFiles, clusterIDToLinkTitles, clusterIDToFileList, \
                               clusterIDToRepFile, clusterIDToImageTag, fileNameToRank, numFeaturesToShow, name, removedFiles, \
                               False), "../webPages/"+firstname+"_"+lastname+"_pp.html")
    #make non pretty print page
    writeStringToFile(makeHTML(clusterIDToBigrams, clusterIDToEmployHist, clusterIDToEduHist, clusterIDToEmails, \
                               clusterIDToMiddleName, clusterIDToSelectedFiles, clusterIDToLinkTitles, clusterIDToFileList,\
                               clusterIDToRepFile, clusterIDToImageTag, fileNameToRank, numFeaturesToShow, name, removedFiles, \
                               True), "../webPages/"+firstname+"_"+lastname+".html")
