"""
copyright ICSI
"""

import sys
import os
import re
from utils import *
from operator import itemgetter
  
"""
input: url, last name
output: title of page if title exists, page has the last name, and url can be opened; "" otherwise
"""
def getPageTitle(url, lastname):
    title = ""
    try:
        page = getURLPage(url)
        titlebegin = re.compile("<title>", re.IGNORECASE).search(page)
        titleend = re.compile("</title>", re.IGNORECASE).search(page)
        if titlebegin != None and titleend != None and re.compile(lastname, re.IGNORECASE).search(page) != None:
            titleBeginPos = page.find(titlebegin.group(0))
            titleEndPos = page.find(titleend.group(0))
            title = removeExtraWhiteSpace(page[titleBeginPos+7:titleEndPos])
    except:
        print "Unexpected error:", sys.exc_info()[0], " on file: "+url
    return title

"""
input: list of urls for the cluster, dict with key = url, value = querry rank, last name
output: The url with the lowest rank for that cluster and the url's title
"""
def repFileFinder(fileList, fileNameToRank, lastname):
    bestRank = 9999
    bestFile = ""
    for filename in fileList:
        if not fileNameToRank.has_key(filename):
            fileNameToRank[filename] = 100
        if bestRank > fileNameToRank[filename]:
            bestRank = fileNameToRank[filename]
            bestFile = filename
    bestFileTitle = getPageTitle("http://"+bestFile, lastname)
    if bestFileTitle == "":
        return [bestFile, bestFile]
    return [bestFile, bestFileTitle]

"""
input: links file, first name, last name
output: dict with key = link url and value = url title
"""
def processPageLinks(linksfile, firstname, lastname):
    linkToTitle = {}
    i = 0
    processedpages = []
    gottentitles = []
    for link in wordFileToList(linksfile):
        if processedpages.count(link) == 0 and re.compile("http://", re.IGNORECASE).match(link) != None:
            link = link.replace("&amp;", "&")
            link = link.replace("&#38;", "&")
            link = link.replace("&quot;", "\"")
            link = link.replace("&#34;", "\"")
            processedpages.append(link)
            title = getPageTitle(link, lastname)
            if title != "" and gottentitles.count(title) == 0:
                linkToTitle[link] = title
                gottentitles.append(title)
                i = i+1
                if i > 4: return linkToTitle
    return linkToTitle

"""
input: html file, file url, last name
output: an image tag
"""
def findImgInHTML(html, url, lastname):
    urlendRE = re.compile(r'/[^/]*$')
    imgStartMatches = re.compile("<img\s+([^>]+)>", re.IGNORECASE).finditer(html)
    for imgStartMatch in imgStartMatches:
        tempURL = url
        if imgStartMatch != None:
            imgSRC = re.compile("src\s*=\s*(\'|\")([^\'\"]+)(\'|\")", re.IGNORECASE).search(imgStartMatch.group(1))
            if imgSRC != None:
                src = imgSRC.group(2)
                if re.compile("http", re.IGNORECASE).search(src) != None:
                    urlend = urlendRE.search(src)
                    if urlend != None:
                        if re.compile(lastname, re.IGNORECASE).search(urlend.group(0)) != None:
                            return "<img src=\""+src+"\" height=100>"
                else:
                    if src[0] == "/":
                        src = src[1:]
                        urlSlashPos = tempURL.find("/")
                        if urlSlashPos != -1:
                            tempURL = tempURL[:urlSlashPos]               
                    else:
                        urlend = urlendRE.search(tempURL)
                        if urlend != None:
                            if re.compile("[.]",re.IGNORECASE).search(urlend.group(0)) != None:
                                tempURL = tempURL.replace(urlend.group(0), "")
                    srcendMatch = urlendRE.search(src)
                    if srcendMatch != None:
                        srcend = srcendMatch.group(0)
                    else:
                        srcend = src                    
                    if re.compile(lastname, re.IGNORECASE).search(srcend) != None:
                        return "<img src=\"http://"+tempURL+"/"+src+"\" height=100>"
    return ""

"""
input: html file, file url, last name, list of homepages, path to data, list of urls in cluster
output: an image tag if the html file contains an image tag
"""
def imagegetter(htmlfilepath, url, lastname, homepages, pathPrefix, fileList):
    imageTag = ""
    #search rep file for image
    html = readFile(htmlfilepath)
    imageTag = findImgInHTML(html, url, lastname)
    if imageTag != "": return imageTag
    #search homepages for image
    for filename in fileList:
        if filename in homepages and filename != url:
            html = readFile(pathPrefix+"/links/"+urlToOsSafeURL("http://"+filename))
            imageTag = findImgInHTML(html, url, lastname)
            if imageTag != "": return imageTag
    return imageTag

"""
input: word, word List
output: true if word is part of a word in word list, ignoring case; false otherwise
"""
def wordInWordList(word, wordList):
    for wordInList in wordList:
        if re.compile(" "+word+" ", re.IGNORECASE).search(" "+wordInList+" ") != None:
            return True
    return False
        
"""
input: bigram dict, list of words between barriers, number of bigrams to show on webpage
Combine gram that ends with the same word that another gram begins with if the new gram is in the list of words passed in
Add the new gram with the counts = minimum counts of the combined grams
Set the counts for the old grams to the old counts - the counts of the new gram
"""
def combineNGrams(bigramCounter, wordList, numFeaturesToShow):
   #Get the top 2*numFeaturesToShow bigrams with the highest counts from the bigram dict
   topBigramList = sorted(bigramCounter.items(), key=itemgetter(1), reverse=True)[0:2*numFeaturesToShow]
   #Remove these bigrams from the bigram dict
   for pair in topBigramList:
       del bigramCounter[pair[0]]
   clusteringOccured = 1
   while clusteringOccured == 1:
       clusteringOccured = 0
       for a, tup1 in enumerate(topBigramList):
           for b, tup2 in enumerate(topBigramList):
               if a != b and tup2[1] != [] and tup1[1] != []:
                   featurelist1 = tup1[0].split()  #Split gram into a list
                   featurelist2 = tup2[0].split()  #Split gram into a list
                   count1 = tup1[1]  #Get the gram counts
                   count2 = tup2[1]  #Get the gram counts
                   #check if gram1 ends with the same word as gram2 begins with
                   if re.compile(featurelist1[-1]+"$", re.IGNORECASE).match(featurelist2[0]):  
                       newFeature = listToString(featurelist1) + " " + listToString(featurelist2[1:])
                       if wordInWordList(newFeature, wordList):  #check if combined grams are in word list
                           clusteringOccured = 1
                           if count1[0] == count2[0]:
                               topBigramList[b] = (newFeature, count1)
                               topBigramList[a] = ("", [])
                           elif count1[0]>count2[0]:
                               topBigramList[b] = (newFeature, count2)
                               topBigramList[a] = (tup1[0], [count1[0]-count2[0], count1[1]-count2[1]])
                           else:
                               topBigramList[a] = (newFeature, count1)
                               topBigramList[b] = (tup2[0], [count2[0]-count1[0], count2[1]-count1[1]])
                           break
           if clusteringOccured == 1:
               break
   for pair in topBigramList:
       if pair[1] != []:
           bigramCounter[pair[0]] = pair[1]

"""
input: list of urls in cluster, last name, path to link file of the rep file, list of homepages, path to data
output: dict with wikipedia, myspace, and flikr files in the cluster or linked to by the rep file
"""
def findSelectedFiles(fileList, lastname, repFileLinksFile, homepages, pathPrefix):
    selectedFilesDict = {}
    foundWiki = False
    foundMySpace = False
    foundFlickr = False
    #search urls for selected files
    for url in fileList:
        if not foundWiki and re.compile("(en|www)[.]wikipedia[.]org", re.IGNORECASE).search(url) != None:
            if re.compile(lastname, re.IGNORECASE).search(url) != None:
                selectedFilesDict["Wikipedia"] = url
                foundWiki = True
        elif not foundMySpace and re.compile("www[.]myspace[.]com", re.IGNORECASE).search(url) != None:
            selectedFilesDict["My Space"] = url
            foundMySpace = True
    #search repfile for links to selected files
    linksList = wordFileToList(repFileLinksFile)
    for link in linksList:
        if not foundWiki and re.compile("(en|www)[.]wikipedia[.]org", re.IGNORECASE).search(link) != None:
            if re.compile(lastname, re.IGNORECASE).search(link) != None:
                selectedFilesDict["Wikipedia"] = link[7:]
                foundWiki = True
        elif not foundMySpace and re.compile("www[.]myspace[.]com", re.IGNORECASE).search(link) != None:
            selectedFilesDict["My Space"] = link[7:]
            foundMySpace = True
        elif not foundFlickr and re.compile("[.]flickr[.]com", re.IGNORECASE).search(link) != None:
            selectedFilesDict["Flickr"] = link[7:]
            foundFlickr = True
    #search homepages for links to selected files
    for url in fileList:
        if url in homepages:
            linksList = wordFileToList(pathPrefix+"/links/"+urlToOsSafeURL("http://"+url))
            for link in linksList:
                if not foundWiki and re.compile("(en|www)[.]wikipedia[.]org", re.IGNORECASE).search(url) != None:
                    if re.compile(lastname, re.IGNORECASE).search(url) != None:
                        selectedFilesDict["Wikipedia"] = url
                        foundWiki = True
                    elif not foundMySpace and re.compile("www[.]myspace[.]com", re.IGNORECASE).search(url) != None:
                        selectedFilesDict["My Space"] = url
                        foundMySpace = True
                    elif not foundFlickr and re.compile("[.]flickr[.]com", re.IGNORECASE).search(url) != None:
                        selectedFilesDict["Flickr"] = url
                        foundFlickr = True
    return selectedFilesDict

def postCluster(clusterIDToFileList, clusterIDToBigrams, clusterIDToWordSetList, clusterIDToRepFile, clusterIDToImageTag, \
                clusterIDToSelectedFiles, clusterIDToLinkTitles, fileNameToRank, firstname, lastname, pathPrefix, \
                numFeaturesToShow, homepages):
    removedFiles = []  #List of urls removed

    for clusterID in clusterIDToFileList:
        #remove clusters that don't have bigrams or have 1 file and that file is not a homepage
        if len(clusterIDToBigrams[clusterID]) < 1 or \
               (len(clusterIDToFileList[clusterID]) < 2 and not (clusterIDToFileList[clusterID][0] in homepages)):
            removedFiles.extend(clusterIDToFileList[clusterID])
        else:
            #Find representative file and url title
            clusterIDToRepFile[clusterID] = repFileFinder(clusterIDToFileList[clusterID], fileNameToRank, lastname)  
            repFileLinksFile = pathPrefix+"/links/"+urlToOsSafeURL("http://"+clusterIDToRepFile[clusterID][0]) #Links file for the rep file
            repFileHTMLFile = pathPrefix+"/html/"+urlToOsSafeURL("http://"+clusterIDToRepFile[clusterID][0])  #HTML file for the rep file
            #Get the titles for the links in the rep file
            clusterIDToLinkTitles[clusterID] = processPageLinks(repFileLinksFile, firstname, lastname)
            #Get an image from the rep file
            clusterIDToImageTag[clusterID] = imagegetter(repFileHTMLFile, clusterIDToRepFile[clusterID][0], lastname, homepages, \
                                                         pathPrefix, clusterIDToFileList[clusterID])
            #Combine bigrams
            combineNGrams(clusterIDToBigrams[clusterID], clusterIDToWordSetList[clusterID], numFeaturesToShow)
            #find selected files
            clusterIDToSelectedFiles[clusterID] = findSelectedFiles(clusterIDToFileList[clusterID], lastname, repFileLinksFile, \
                                                                    homepages, pathPrefix)
    return removedFiles
