"""
copyright ICSI
"""

import sys
import os
import re
from utils import *
from processInfo import *
from processBigrams import *
from processLinks import *
from preProcessPage import *

"""
extract bigrams, links, and other information from files
"""
def preCluster(filenames, pathPrefix, homepages, firstname, lastname, stopWordList, clusterIDToFileList, clusterIDToBigrams, \
           clusterIDToWordSetList, clusterIDToLinks, clusterIDToEmployHist, clusterIDToEduHist, clusterIDToEmails, clusterIDToMiddleName):
    namePassage = re.compile("(\s"+lastname+"[^a-zA-Z0-9_\&\.]?[\s]*([\S]+[\s]+){5,50})", re.IGNORECASE) #content to search if not homepage
    
    clusterID = 0
    for filename in filenames:
        if hasBadURLTerms(filename): continue  #if filename has term from badurls.txt, ignore file

        if isSelectedFile(osSafeURLToURL(filename)):  #process the fileContent and links differently if it is a selected url
            fileContent = preProcessSelectedURLText(readFile(pathPrefix+"/text/"+filename), osSafeURLToURL(filename), lastname)
            fileLinks = []
        else:            
            fileContent = readFile(pathPrefix+"/text/"+filename)
            fileLinks = wordFileToList(pathPrefix+"/links/"+filename)
        fileHTML = readFile(pathPrefix+"/html/"+filename)  #load file html

        filename = osSafeURLToURL(filename)[7:]  #convert filename to url
        if filename not in homepages and \
           nameNotInPage(firstname, lastname, fileContent): continue  #if file is not a homepage and doesn't contain the name, ignore file
        if wikiReference(fileContent): continue  #if file has a link to wikipedia ignore it
        fileContent = removeNotText(fileContent, lastname)
    
        bigramCounter = {}  #(dict with key = bigram, value = [# of times it appears in cluster, # of clusters it appears in])
        wordSetList = []
        if filename in homepages:  #if homepage process all file text
            #extract bigrams and word sets
            bigramCounter, wordSetList = extractBigramFeatureCounts(preProcessForBigramExtraction(fileContent), stopWordList)  
        else:   #else process only matches to namePassage RE
            for match in namePassage.findall(fileContent):
                preProcessContent = preProcessForBigramExtraction(match[0])
                bigramCounter2, wordSetList2 = extractBigramFeatureCounts(preProcessContent, stopWordList)
                bigramCounter = bigramDictUnion(bigramCounter, bigramCounter2)
                wordSetList = listUnion(wordSetList, wordSetList2)

        fileLinks.append(filename)  #add file url to list of links
        clusterIDToLinks[clusterID] = makeLinksDict(fileLinks, lastname)  #store file links
        clusterIDToBigrams[clusterID] = removeBadBigrams(bigramCounter)  #store file bigrams
        clusterIDToWordSetList[clusterID] = wordSetList  #store file word sets
        clusterIDToFileList[clusterID] = [filename]   #store file name

        #If there are more than 20 bigrams with coutns greater than 2, remove bigrams with counts less than 3
        if bigramCountsGreaterThan(clusterIDToBigrams[clusterID], 2) > 20:
            removeBigramCountsLessThan(clusterIDToBigrams[clusterID], 3)
    
        #extract file information
        clusterIDToEmails[clusterID], clusterIDToMiddleName[clusterID], clusterIDToEmployHist[clusterID], clusterIDToEduHist[clusterID] = \
                                      extractInfo(preProcessForInfoExtraction(fileContent), fileHTML, filename, firstname, lastname)
        clusterID = clusterID + 1
