"""
copyright ICSI
"""

import re
from utils import *

"""
input: url
output: true if url is a myspace, wikipedia, or flikr url
"""
def isSelectedFile(url):
    if re.compile("([.]wikipedia[.]org|www[.]myspace[.]com)", re.IGNORECASE).search(url) != None:
        return True
    return False

"""
input: file text, last name
output: text with non-useful information removed
"""
def preProcessWikiText(fileContent, lastname):
    textStart = re.compile(r''+lastname+'\s+[(].*\d{2,}.*[)]', re.IGNORECASE).search(fileContent)
    startPos = 0
    if textStart != None:
        startPos = fileContent.find(textStart.group(0))       
    textEnd = re.compile(r'external links'+'[ ]*\n+[ ]*'+'([*].+\n[ ]*)+\n', re.IGNORECASE).search(fileContent)
    endPos = -1
    if textEnd != None:
        endPos = fileContent.rfind(textEnd.group(0))+len(textEnd.group(0))
    return fileContent[startPos:endPos]

"""
input: file text
output: text with non-useful information removed
"""
def preProcessMySpaceText(fileContent):
    textStart = re.compile(r'home\s+[|]\s+((\S+\s+)+[|]\s+)+classifieds', re.IGNORECASE).search(fileContent)
    startPos = 0
    if textStart != None:
        startPos = fileContent.find(textStart.group(0))+len(textStart.group(0))     
    textEnd = re.compile(r'about\s+[|]\s+((\S+\s+)+[|]\s+)+myspace international', re.IGNORECASE).search(fileContent)
    endPos = -1
    if textEnd != None:
        endPos = fileContent.rfind(textEnd.group(0))
    return fileContent[startPos:endPos]

"""
input: file text, url, last name
output: text with non-useful information removed
"""
def preProcessSelectedURLText(fileContent, url, lastname):
    if re.compile("wikipedia", re.IGNORECASE).search(url) != None:
        return preProcessWikiText(fileContent, lastname)
    elif re.compile("myspace", re.IGNORECASE).search(url) != None:
        return preProcessMySpaceText(fileContent)
"""
input: file text
output: true if text contains the words "referenced from", "reference article from", or "retrieved from" in the same sentence
"""
def wikiReference(fileContent):
    wikiRefRE = re.compile(r'(\s|^)(referenced|reference article|retrieved)\s+from((\s+\S+){5})', re.IGNORECASE)
    wikiRefMatches = wikiRefRE.finditer(fileContent)
    for wikiRefMatch in wikiRefMatches:
        if wikiRefMatch != None:
            if re.compile("wikipedia", re.IGNORECASE).search(wikiRefMatch.group(3)) != None:
                return True
    return False

"""
input: firstname, lastname, file text
output: true if first name, possible middle name, and last name appear in order in the file
"""
def nameNotInPage(firstname, lastname, fileContent):
    regPost = r'[\'\"]'
    regPre = r'[\'\"]'
    regWord = r'('+regPre+'?('+regL+'+|'+regL+'+'+regI+regL+'+)'+regPost+'?)'	
    middleNameRE = re.compile(r'(\s|^)'+firstname+'(('+regS+regWord+'|'+regS+'\w[.]){1,2})?'+regS+lastname, re.IGNORECASE)
    if middleNameRE.findall(fileContent) == []:
        return True
    return False

"""
input: file text, last name
output: file text with all sentences containing "not" and the last name removed
"""
def removeNotText(fileContent, lastname):
    sentRE = re.compile(r'(^|\n|[.])(('+regS+regWord+')+)[!.]', re.IGNORECASE)
    sentMatches = sentRE.finditer(fileContent)
    for sentMatch in sentMatches:
        if sentMatch != None:
            posNotSent = sentMatch.group(2)
            if re.compile(r' not ', re.IGNORECASE).findall(" "+posNotSent+" ") != [] and \
                   re.compile(r''+regS+'('+regPre+'?('+lastname+')'+regPost+'?)'+regS+'', re.IGNORECASE).findall(" "+posNotSent+" ") != []:
                fileContent = fileContent.replace(posNotSent, "")
    return fileContent

"""
input: file content
output: file content preprocessed for information extraction
"""
def preProcessForInfoExtraction(fileContent):
    fileContent = re.compile(r'(\s|^)(\w)[.](\w)[.](\s|^)', re.IGNORECASE).sub(r'\g<1>\g<2>_\g<3>\g<4>', fileContent)
    fileContent = re.compile(r'(\s|^)co[.](\s|$|,)', re.IGNORECASE).sub(r'\g<1>Corporation\g<2>', fileContent)
    fileContent = re.compile(r'(\s|^)inc[.](\s|$|,)', re.IGNORECASE).sub(r'\g<1>Incorporated\g<2>', fileContent)
    fileContent = re.compile(r'(\s|^)prof[.](\s|$|,)', re.IGNORECASE).sub(r'\g<1>Professor\g<2>', fileContent)
    fileContent = re.compile(r'(\s|^)tech[.](\s|$|,)', re.IGNORECASE).sub(r'\g<1>Technology\g<2>', fileContent)
    fileContent = re.compile(r'(\s|^)inst[.](\s|$|,)', re.IGNORECASE).sub(r'\g<1>Institute\g<2>', fileContent)
    fileContent = re.compile(r'((\s|^)(b|m))[.](a)[.](\s|$|,)',re.IGNORECASE).sub(r'\g<1>\g<4>', fileContent)
    fileContent = re.compile(r'(\s|^)ph[.]d[.](\s|$|,)', re.IGNORECASE).sub(r'\g<1>PhD\g<2>', fileContent)
    fileContent = re.compile(r'(\s|^)\[.+[.](jpg|jpeg|gif|bmp|tiff|png|wmf|pcx)\](\s|$)', re.IGNORECASE).sub(r'\g<1>\g<3>', fileContent)
    return fileContent

"""
input: file content
output: file content preprocessed for bigram extraction
"""
def preProcessForBigramExtraction(content):
    content = content.replace("(", " *LPAREN* ")
    content = content.replace("=", " *EQUAL* ")
    content = content.replace("<", " *LT* ")
    content = content.replace(">", " *GT* ")
    content = content.replace("\'", " *SINGLEQUOTE* ")
    content = content.replace("\"", " *DOUBLEQUOTE* ")
    content = content.replace(",", " *COMMA* ")
    content = content.replace(")", " *RPAREN* ")
    content = content.replace(":", " *COLON* ")
    content = content.replace("?", " *QUESTIONMARK* ")
    content = content.replace("!", " *EXCLAMATIONPOINT* ")
    content = content.replace("|", " *PIPE* ")
    content = content.replace("\"", " *LSLASH* ")
    content = content.replace("[", " *LBRACKET* ")
    content = content.replace("]", " *RBRACKET* ")
    content = content.replace("--", " *DOUBLEDASH* ")
    content = content.replace(";", " *SEMICOLON* ")
    content = content.replace("/", " *RSLASH* ")
    content = content.replace("http", " *HTTP* ")
    content = content.replace("HTTP", " *HTTP* ") 
    content = content.replace(".", " *PERIOD* ")
    content = content.replace("\n", " *NEWLINE* ")
    content = content.replace("&", " *AMP*")
    return content
