"""
copyright ICSI
"""

import sys
import os
from utils import *

jobRE = wordFileToRE("../lists/jobs.txt")

"""
Looks for appositives following the last name in the text (i.e. last name, set of words,)
Looks for these patterns in the appositives:
1.      , (8 word profession description) ("at" or "for") (8 word employer description),
2.      , (8 word profession description),
"""
def processAppo(employHistList, fileContent, filename, lastname):
	regPost = r'[)>\]}\'\";:]'
	regWord = r'(('+regPre+'?('+regL+'+|'+regL+'+'+regI+regL+'+)'+regPost+'?)|'+regC+')'
	regEnd = r'[!.,]'
	appoRE = re.compile(r'(\s|^)'+lastname+',(('+regS+regWord+'){2,}?)'+regEnd, re.IGNORECASE)
	appoMatches = appoRE.finditer(fileContent)
	for appoMatch in appoMatches:
		if appoMatch != None:
			infoRE = re.compile(r'('+regS+'(who|is|a|the|an))*(('+regS+regWord+'){1,8}?)'+regS+'(at|for)(('+\
								regS+regWord+'){1,8})$', re.IGNORECASE)
			infoMatch = infoRE.match(appoMatch.group(2))
			if infoMatch != None:
				posProfession = removeExtraWhiteSpace(infoMatch.group(3))
				posEmployer = removeExtraWhiteSpace(infoMatch.group(9))
				if re.compile(r''+regS+'('+regPre+'?('+jobRE+')'+regPost+\
					      '?)'+regS+'', re.IGNORECASE).findall(" "+posProfession+" ") != []:
					employHistList.append({"Profession":posProfession, "Employer":posEmployer, \
							       "URLs":[filename]})
					continue
			infoRE = re.compile(r'('+regS+'(who|is|a|the|an))*(('+regS+regWord+'){1,8})$', re.IGNORECASE)
			infoMatch = infoRE.match(appoMatch.group(1))
			if infoMatch != None:
				posProfession = removeExtraWhiteSpace(infoMatch.group(3))
				if re.compile(r''+regS+'('+regPre+'?('+jobRE+')'+regPost+\
					      '?)'+regS+'', re.IGNORECASE).findall(" "+posProfession+" ") != []:
					employHistList.append({"Profession":posProfession, "URLs":[filename]})

"""
Looks for title blocks after the full name in the text
(i.e.
      first and last name
      1-8 words
      1-8 words
      1-8 words
)      
Looks for these patterns in the title block lines:
1.      8 word profession description
2.      8 word degree description
3.      8 word employer description
4.      8 word department description
"""
def processTitleBlock(employHistList, eduHistList, fileContent, filename, firstname, lastname):
	titleBlockRE1 = re.compile(r'(\n|^)[ ]*'+'('+firstname+'(('+regS+regWord+'|'+regS+'\w[.]){1,2})?'+regS+\
		     lastname+'('+regS+regPre+regEmail+regPost+')?[ ]*(\n[ ]*)+)+(((\S+('+regS+'\S+){,7})[ ]*\n[ ]*){2,5})', re.IGNORECASE)
	titleBlockMatches = titleBlockRE1.finditer(fileContent)
	for titleBlockMatch in titleBlockMatches:
		if titleBlockMatch != None:
			titleBlockFound = titleBlockMatch.group(10)
			tempEmployHistList = {}
			tempEduHistList = {}
			for lineNum, titleBlockLine in enumerate(titleBlockFound.split("\n")):
				if re.compile(r'[ ]*('+regWord+'('+regS+regWord+')*)[ ]*$', re.IGNORECASE).match(titleBlockLine) != None:
					if lineNum == 0 and re.compile(r'('+regS+'|'+regPre+')('+jobRE+')('+\
							      regS+'|'+regPost+')', re.IGNORECASE).findall(" "+titleBlockLine+" ") != []:
							tempEmployHistList["Profession"] = removeExtraWhiteSpace(titleBlockLine)
					elif re.compile(r'('+regS+'|'+regPre+')(ba|ma|phd)('+regS+'|'+regPost+\
							')', re.IGNORECASE).findall(" "+titleBlockLine+" ") != []:
						if tempEduHistList.has_key("Degree"):
							tempEduHistList["Degree"].append(removeExtraWhiteSpace(titleBlockLine))
						else:
							tempEduHistList["Degree"] = [removeExtraWhiteSpace(titleBlockLine)]
					elif re.compile(r'('+regS+'|'+regPre+')(university|institute|college)('+\
							       regS+'|'+regPost+')', re.IGNORECASE).findall(" "+titleBlockLine+" ") != []:
						tempEmployHistList["Employer"] = removeExtraWhiteSpace(titleBlockLine)
						tempEmployHistList["Profession"] = "Professor"
					elif re.compile(r'('+regS+'|'+regPre+')(department|chair'+regS+'of|division)('+\
							regS+'|'+regPost+')', re.IGNORECASE).findall(" "+titleBlockLine+" ") != []:
						if tempEmployHistList.has_key("Department"):
							tempEmployHistList["Department"].append(removeExtraWhiteSpace(titleBlockLine))
						else:
							tempEmployHistList["Department"] = [removeExtraWhiteSpace(titleBlockLine)]
			if tempEmployHistList != {}:
				tempEmployHistList["URLs"] = [filename]
				employHistList.append(tempEmployHistList)
			if tempEduHistList != {}:
				tempEduHistList["URLs"] = [filename]
				eduHistList.append(tempEduHistList)

"""
input: email
output: regularied email with evertying after the email name and before the domain replaced with "@"
"""
def regularizeEmail(email):
	emailRE1 = re.compile(r'(.+?)[^a-zA-Z0-9]?a[^a-zA-Z0-9]?t[^a-zA-Z0-9]?(.+?)([?]|$)', re.IGNORECASE)
	emailRE2 = re.compile(r'(.+?)[(]a[)](.+?)([?]|$)', re.IGNORECASE)
	email = email.replace(" ", "")
	emailMatch = emailRE1.match(email)
	if emailMatch != None:
		email = emailMatch.group(1)+"@"+emailMatch.group(2)
	emailMatch = emailRE2.match(email)
	if emailMatch != None:
		email = emailMatch.group(1)+"@"+emailMatch.group(2)	
	return email

"""
Looks for the first and last name followed by an "<A HREF="mailto:" tag in the html code
"""
def processEmail(emailList, html, filename, firstname, lastname):
	regPost = r'[\'\"]'
	regPre = r'[\'\"]'
	regWord = r'('+regPre+'?('+regL+'+|'+regL+'+'+regI+regL+'+)'+regPost+'?)'
	emailRE = re.compile(r'(\s|^)'+firstname+'((\s+'+regWord+'|\s+\w[.]){1,2})?\s+'+lastname+\
							'[^\n]{,5}?<A HREF="mailto:(.+)">', re.IGNORECASE)
	emailMatches = emailRE.finditer(html)
	for emailMatch in emailMatches:
		if emailMatch != None:
			emailList.append({"Email":regularizeEmail(emailMatch.group(6)), "URLs":[filename]})

"""
Looks for the first and last name preceeded by a 1 word profession
"""
def processTitleBeforeName(employHistList, fileContent, filename, firstname, lastname):
	regPost2 = r'[\'\"]'
	regPre2 = r'[\'\"]'
	regWord2 = r'('+regPre2+'?('+regL+'+|'+regL+'+'+regI+regL+'+)'+regPost2+'?)'
	preNameRE = re.compile(r'(\s|^)('+jobRE+')'+regS+firstname+\
							'(('+regS+regWord2+'|'+regS+'\w[.]){1,2})?'+regS+lastname, re.IGNORECASE)
	preNameMatches = preNameRE.finditer(fileContent)
	for preNameMatch in preNameMatches:
		if preNameMatch != None:
			employHistList.append({"Profession":removeExtraWhiteSpace(preNameMatch.group(2)), "URLs":[filename]})

"""
Looks for sentences that include the last name in the text (i.e. last name + set of words.)
Looks for these patterns in the sentences:
1.      (8 word profession description) ("in") (8 word department description) ("at" or "for") (8 word employer description).
2.      ("joined") (8 word employer description) ("as") (8 word profession description).
"""
def processFullSentence(employHistList, fileContent, filename, lastname):
	fullSentRE1 = re.compile(r'(\s|^)'+lastname+'(,'+regS+'\d{2},)?(('+regS+regWord+'){3,}?)'+regEnd, re.IGNORECASE)
	fullSentMatches = fullSentRE1.finditer(fileContent)
	for fullSentMatch in fullSentMatches:
		if fullSentMatch != None:
			infoRE = re.compile(r''+regS+'is('+regS+'(a|the|an))?(('+regS+regWord+'){1,8}?)('+regS+\
					    'in(('+regS+regWord+'){1,8}?))?'+regS+'(at|for)(('+\
					    regS+regWord+'){1,8}?)($|'+regS+'and'+regS+')', re.IGNORECASE)
			infoMatch = infoRE.match(fullSentMatch.group(3))
			if infoMatch != None:
				posProfession = removeExtraWhiteSpace(infoMatch.group(3))
				posEmployer = removeExtraWhiteSpace(infoMatch.group(15))
				if re.compile(r''+regS+'('+regPre+'?('+jobRE+')'+regPost+'?)'+regS, re.IGNORECASE).findall(" "+\
												  posProfession+" ") != []:
					employHistList.append({"Profession":posProfession, "Employer":posEmployer, "URLs":[filename]})
					if infoMatch.group(8) != None:
						if re.compile(r''+regS+'('+regPre+'?(department|chair'+regS+'of|division)'+regPost+'?)'+\
						   regS+'',re.IGNORECASE).findall(" "+removeExtraWhiteSpace(infoMatch.group(8))+" ") != []:
							employHistList[-1]["Department"] = [infoMatch.group(8)]
					continue
			infoRE = re.compile(r''+regS+'(joined|joins|has'+regS+'joined|is'+regS+'joining|will'+regS+'join)(('+regS+\
					    regWord+'){1,8}?)'+regS+'as(('+regS+regWord+'){1,8}?)($|'+regS+'and'+regS+')', re.IGNORECASE)
			infoMatch = infoRE.match(fullSentMatch.group(3))
			if infoMatch != None:
				posProfession = removeExtraWhiteSpace(infoMatch.group(7))
				posEmployer = removeExtraWhiteSpace(infoMatch.group(2))
				if re.compile(r''+regS+'('+regPre+'?('+jobRE+')'+regPost+'?)'+regS+'', re.IGNORECASE).findall(" "+\
													    posProfession+" ") != []:
					employHistList.append({"Profession":posProfession, "Employer":posEmployer, \
							       "URLs":[filename]})

"""
Looks for the first name followed by a 1-2 word middle name and the last name
The middle name must not contain the first or last name or the word "and"
"""
def processMiddleName(middlenameList, fileContent, filename, firstname, lastname):
	regPost = r'[\'\"]'
	regPre = r'[\'\"]'
	regWord = r'('+regPre+'?('+regL+'+|'+regL+'+'+regI+regL+'+)'+regPost+'?)'	
	middleNameRE = re.compile(r'(\s|^)'+firstname+'(('+regS+regWord+'|'+regS+'\w[.]){1,2})?'+regS+lastname, re.IGNORECASE)
	middleNameMatches = middleNameRE.finditer(fileContent)
	for middleNameMatch in middleNameMatches:
		if middleNameMatch != None:
			if middleNameMatch.group(2) != None:
				posMiddleName = removeExtraWhiteSpace(middleNameMatch.group(2))
				if re.compile(firstname, re.IGNORECASE).findall(posMiddleName) == [] and \
				       re.compile(lastname, re.IGNORECASE).findall(posMiddleName) == [] and \
				       re.compile(" and ", re.IGNORECASE).findall(" "+posMiddleName+" ") == []:
					middlenameList.append({"Middle Name":posMiddleName, "URLs":[filename]})

"""
input: file text, html, file name, first name, last name
output:
email list  (i.e. list of {"Email": ab@cd.com, "URLs": [www.cd.com]})
middlename list  (i.e. list of {"Middle Name": John, "URLs": [www.cd.com]})
employment history list  (i.e. list of {"Profession": Manager, "Employer": cd, "URLs": [www.cd.com]})
education history list  (i.e. list of {"Degree": B.S. Computer Science, "URLs": [www.cd.com]})
"""
def extractInfo(fileContent, html, filename, firstname, lastname):
	emailList = []
	middlenameList = []
	employHistList = []
	eduHistList = []

	processAppo(employHistList, fileContent, filename, lastname)
	processTitleBlock(employHistList, eduHistList, fileContent, filename, firstname, lastname)
	processEmail(emailList, html, filename, firstname, lastname)
	processTitleBeforeName(employHistList, fileContent, filename, firstname, lastname)
	processFullSentence(employHistList, fileContent, filename, lastname)
	processMiddleName(middlenameList, fileContent, filename, firstname, lastname)
	return emailList, middlenameList, employHistList, eduHistList

"""
input: 2 middle name lists
output: true if there is a common middle name between the lists
"""
def commonMiddleName(middlenameList1, middlenameList2):
	for dict1 in middlenameList1:
		for dict2 in middlenameList2:
		    if dict1["Middle Name"] == dict2["Middle Name"]:
			    return True
	return False
		    
"""
input: 2 email lists
output: true if there is a common email between the lists
"""
def commonEmail(emailList1, emailList2):
	for dict1 in emailList1:
		for dict2 in emailList2:
		    if dict1["Email"] == dict2["Email"]:
			    return True
	return False

"""
input: 2 middle name lists
adds the values of the 2nd middle name list to the first
"""
def middleNameDictUnion(middleNameList1, middleNameList2):
	foundMatch = False
	for dict2 in middleNameList2:
		for dict1 in middleNameList1:
			if dict1["Middle Name"] == dict2["Middle Name"]:
				foundMatch = True
				dict1["URLs"].extend(dict2["URLs"])
				break
		if not foundMatch:
			middleNameList1.append(dict2)

"""
input: 2 email lists
adds the values of the 2nd email list to the first
"""
def emailDictUnion(emailList1, emailList2):
	foundMatch = False
	for dict2 in emailList2:
		for dict1 in emailList1:
			if dict1["Email"] == dict2["Email"]:
				foundMatch = True
				dict1["URLs"].extend(dict2["URLs"])
				break
		if not foundMatch:
			emailList1.append(dict2)
"""
input: 2 information dicts, key
output: [True if one dict value contains all the words in the other dict value, value]; otherwise returns [False, None]
"""
def valueOverlap(infoDict1, infoDict2, key):
	if not infoDict1.has_key(key) and not infoDict2.has_key(key):
		return True, None
	elif not infoDict1.has_key(key):
		return True, infoDict2[key]
	elif not infoDict2.has_key(key):
		return True, infoDict1[key]
	else:
		info1List = infoDict1[key].split()
		info1InInfo2 = True
		for word in info1List:
			if re.compile(r'('+regS+'|'+regPre+')'+word+'('+regS+'|'+regPost+')', re.IGNORECASE).findall(" "+\
												 infoDict2[key]+" ") == []:
				info1InInfo2 = False
				break
		if info1InInfo2:
			return True, infoDict2[key]
		info2List = infoDict2[key].split()
		info2InInfo1 = True
		for word in info2List:
			if re.compile(r'('+regS+'|'+regPre+')'+word+'('+regS+'|'+regPost+')', re.IGNORECASE).findall(" "+\
												 infoDict1[key]+" ") == []:
				info2InInfo1 = False
				break
		if info2InInfo1:
			return True, infoDict1[key]
	return False, None	

"""
input: 2 employment history lists
adds the values of the 2nd employment history list to the first
"""
def empHistDictUnion(empHistList1, empHistList2):
	foundMatch = False
	for dict2 in empHistList2:
		for dict1 in empHistList1:			
			commonProf, commonProfString = valueOverlap(dict1, dict2, "Profession")
			commonEmp, commonEmpString = valueOverlap(dict1, dict2, "Employer")
			if commonProf and commonEmp:
				foundMatch = True
				if commonProfString != None:
					dict1["Profession"] = commonProfString
				if commonEmpString != None:
					dict1["Employer"] = commonEmpString
				if not dict1.has_key("Department"):
					if dict2.has_key("Department"):
						dict1["Department"] = dict2["Department"]
				dict1["URLs"].extend(dict2["URLs"])
				break
		if not foundMatch:
			empHistList1.append(dict2)

"""
input: 2 education history lists
adds the values of the 2nd education history list to the first
"""
def eduHistDictUnion(eduHistList1, eduHistList2):
	foundMatch = False
	for dict2 in eduHistList2:
		for dict1 in eduHistList1:
			if dict1["Degree"] == dict2["Degree"]:
				foundMatch = True
				dict1["URLs"].extend(dict2["URLs"])
				break
		if not foundMatch:
			eduHistList1.append(dict2)
