"""
copyright ICSI
"""

import sys
import os
import re
from utils import *

badurlterms = wordFileToList("../lists/badurls.txt")  #list of urls to ignore
badLinksList = wordFileToList("../lists/badlinks.txt")  #list of links to ignore

"""
input: url (i.e. "http://citeseer.ist.psu.edu/correct/75125")
output: url with the last "/" and everything after it removed, unless this part has a "~" and the last name
                            (i.e. "citeseer.ist.psu.edu/correct")
"""
def regularizeURL(url, lastname):
    httpRE = re.compile(r'http://', re.IGNORECASE)
    reg_url = re.sub(httpRE,"",url)
    urlendRE = re.compile(r'/[^/]*$')
    urlendMatch = urlendRE.search(reg_url)
    if urlendMatch != None:
        urlendFound = urlendMatch.group(0)
        if re.compile(lastname, re.IGNORECASE).search(urlendFound) == None and \
               re.compile("([.]wikipedia[.]org|www[.]myspace[.]com)", re.IGNORECASE).search(url) == None:
            reg_url = re.sub(urlendRE,"",reg_url)
    return reg_url

"""
input: list of links
output: dict with key = regularized link, value = # of times link appears in cluster
"""
def makeLinksDict(linkList, lastname):
    newDict = {}
    for link in linkList:
        reg_url = regularizeURL(link, lastname)
        if reg_url.strip().lower() not in badLinksList:
            if newDict.has_key(reg_url):
                newDict[reg_url] = newDict[reg_url]+1
            else:
                newDict[reg_url] = 1
    return newDict

"""
input: url
output: true if url contains term in list of bad url terms, false otherwise
"""
def hasBadURLTerms(url):
    for term in badurlterms:
        if url.lower().find(term.lower()) > -1: return True
    return False

"""
input: 2 list of urls
output: true if there is a regularized url in list1 = a regularized url in list2, false otherwise
"""
def commonURL(urlList1, urlList2, lastname):
    for url1 in urlList1:
        for url2 in urlList2:
            if regularizeURL(url1, lastname) == regularizeURL(url2, lastname):
                return True
    return False

"""
input: 2 link dicts
output: # of common links between the dictionaries
"""
def commonLinkCount(linkCounter1, linkCounter2):
    ckc = 0
    for key in linkCounter2:
        if linkCounter1.has_key(key):
            ckc = ckc + 1
    return ckc

"""
input: 2 link dicts
output: the sum of the min value for common links between the dictionaries.
"""
def commonLinkMinSum(linkCounter1, linkCounter2):
    ckms = 0
    for key in linkCounter2:
        if linkCounter1.has_key(key):
            ckms  = ckms + min(linkCounter1[key], linkCounter2[key])
    return ckms

"""
input: 2 link dicts
adds the values of the 2nd link dict to the first
"""
def linkCounterUnion(linkCounter1, linkCounter2):
	for key in linkCounter2:
		if linkCounter1.has_key(key):
			linkCounter1[key] = linkCounter1[key] + linkCounter2[key]
		else:
			linkCounter1[key] = linkCounter2[key]
