"""
copyright ICSI
"""

import sys
import os
import re
import copy
import signal
import urllib

"""
Reg expression definitions
"""
#Letters
regL = r'[a-z]'
#Symbols inbetween letters
regI = r'[-`_&+]'
#Spaces
regS = r'[ ]{1,2}'
#Word Connectors
regC = r'[&+-]'
#Symbols at end of sentence
regEnd = r'[!.\n]'
#Symbols pre-word
regPre = r'[(<\[{\'\"]'
#Symbols post-word
regPost = r'[)>\]}\'\",;:]'
#Email
regEmail = '[^\n]{1,30}'
#Word in sentence
regWord = r'(('+regPre+'?('+regL+'+|'+regL+'+'+regI+regL+'+)'+regPost+'?)|'+regC+')'
#Date in sentence
regDate = r'(('+regPre+'?('+regL+'+|'+regL+'+'+regI+regL+'+)'+regPost+'?)|'+regC+')'

"""
Overrides urlopener to return "", "" when a page asks for a username and password
"""
class myURLOpener(urllib.FancyURLopener):
    def prompt_user_passwd(self, host, realm):
        return "", ""

urlopener = myURLOpener()

"""
Handles timeouts
"""
def handler(signum, frame):
    raise IOError, "Couldn't open device!"

"""
input: url
output: url content
"""
def getURLPage(url):
    try:
        signal.signal(signal.SIGALRM, handler)
        signal.alarm(10)
        f = urlopener.open(url)
        signal.alarm(0)
        page = f.read()
        f.close()
        return page
    except:
        raise IOError, "Couldn't open device!"
"""
input: token
output: true if token has a digit in it, false otherwise
"""
def hasDigitCharacter(token):
    for character in token:
        if character.isdigit(): return True
    return False

"""
input: string
output: string with all whitespaces and newlines replaced with a single whitespace
"""
def removeExtraWhiteSpace(s):
	sList = s.split()
	sNew = ""
	for a in sList:
		sNew = sNew + " " +a
	return sNew.strip()

"""
input: file name
output: content of file
"""
def readFile(inputfilename):
    f = open(inputfilename, "r")
    content =f.read()
    f.close()
    return content

"""
input: file name
output: list with each element being a line from the file content
"""
def wordFileToList(inputfilename):
	inputfile = open(inputfilename, "r")
	wordlist = []
	for line in inputfile:
		line = line.strip()
		wordlist.append(line)
	inputfile.close()
    	return wordlist

"""
input: file name
output: string with and "|" between each line of file content 
"""
def wordFileToRE(inputfilename):
	inputfile = open(inputfilename, "r")
	filestring = ""
	for i, line in enumerate(inputfile):
            line = line.strip()
            if i == 0:
                filestring = filestring + line
            else:
                filestring = filestring + "|" + line
    	return filestring

"""
input: string, file name
output: saves string in file
"""
def writeStringToFile(content, filename):
	f = open(filename, mode="w")
	f.write(content)
	f.close()

"""
input: list
output: string with list elements seperated by a space
"""
def listToString(l):
    listString = ""
    for a in l:
        listString = listString + " " + a
    return listString.strip()

"""
input: url (i.e. "http://www.icsi.berkeley.edu/~jrey/")
output: url with "/" replaced with "-SLASH-" and with final "/" and "http://" removed (i.e. "www.icsi.berkeley.edu-SLASH-~jrey")
"""
def urlToOsSafeURL(url):
    osSafeURL = url[7:]
    if osSafeURL[-1] == "/": osSafeURL = osSafeURL[:-1]
    osSafeURL = osSafeURL.replace("/", "-SLASH-")
    return osSafeURL

"""
input: os safe url (i.e. "www.icsi.berkeley.edu-SLASH-~jrey-SLASH-")
output: url in original form (i.e. "http://www.icsi.berkeley.edu/~jrey/")
"""
def osSafeURLToURL(osSafeURL):
	url = osSafeURL.replace("-SLASH-", "/")
	url = r'http://'+ url
	return url

"""
input: 2 lists
output: a new list with the union of the elements of list1 and list2
"""
def listUnion(l1, l2):
    unionlist = copy.deepcopy(l1)
    for i in l2:
        if i not in unionlist:
            unionlist.append(i)
    return unionlist
